/**
 *  joola.io
 *
 *  Copyright Joola Smart Solutions, Ltd. <info@joo.la>
 *
 *  Licensed under GNU General Public License 3.0 or later.
 *  Some rights reserved. See LICENSE, AUTHORS.
 *
 *  @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
 */

var
  router = require('./index'),
  url = require('url'),

  _datatables = require('../lib/objects/datatables'),
  connector = require('../lib/connectors/connector');

/**
 * Lists all available dimensions. You can provide parameters and ask for a specific data source and datatable.
 *
 * The returned list will contain only dimensions authorized for the authenticated user.
 *
 * @method list
 * @param {string} [datasourceid] Datasource ID.
 * @param {string} [datatableid] Datatable ID.
 * @return {object} json structure listing all dimensions.
 */
exports.index = exports.list = {
  name: 'datatables/list',
  description: 'i list all datatables available',
  inputs: {
    required: [],
    optional: ['datasourceid']
  },
  outputExample: {},
  permission: ['access_system'],
  run: function (req, res) {
    var response = {};
    var datatables = _datatables.list(req.params.datasourceid);
    if (!datatables)
      return router.responseError(new router.ErrorTemplate('Failed to find data table'), req, res);

    response.datatables = ce.clone(datatables);
    return router.responseSuccess(response, req, res);
  }
};

/**
 * Lists all available dimensions. You can provide parameters and ask for a specific data source and datatable.
 *
 * The returned list will contain only dimensions authorized for the authenticated user.
 *
 * @method get
 * @param {string} [dimensionid] Dimension ID.
 * @return {object} json structure holding the dimension details.
 */
exports.get = {
  name: 'datatables/get',
  description: 'i get a specific datatable',
  inputs: {
    required: ['id'],
    optional: []
  },
  outputExample: {},
  permission: ['access_system'],
  run: function (req, res) {
    var response = {};
    var datatable = _datatables.get(req.params.datatableid);
    if (!datatable)
      return router.responseError(new router.ErrorTemplate('Failed to find data table'), req, res);

    response.datatable = ce.clone(datatable);
    return router.responseSuccess(response, req, res);
  }
};

/**
 * Validates the dimension database integrity.
 *
 * @method get
 * @param {string} [dimensionid] Dimension ID.
 * @return {object} json structure holding the validation details.
 */
exports.validate = {
  name: 'datatables/validate',
  description: 'i validate a specific datatable',
  inputs: {
    required: ['datatableid'],
    optional: []
  },
  outputExample: {},
  permission: ['manage_system'],
  run: function (req, res) {
    var response = {};
    var datatable = _datatables.get(req.params.datatableid);
    var query = connector.createQuery();

    response.datatable = ce.clone(datatable);
    response.validated = false;

    query.sql = 'SELECT * FROM ' + datatable.name + ';';
    query.limit = 1;
    query.datasource = datatable.datasource;
    connector.executeQuery(query, function (query, rows, fields, error) {
      response.queryInformation = ce.clone(query);
      if (error)
        throw error;
      else {
        response.validated = true;
      }
      return router.responseSuccess(response, req, res);
    });
  }
};
